/*
 * *      sdadc.c
 * *
 * *      Copyright (C) 2012 Jiri Pittner <jiri@pittnerovi.com>
 * *
 * *      This program is free software: you can redistribute it and/or modify
 * *      it under the terms of the GNU General Public License as published by
 * *      the Free Software Foundation, either version 3 of the License, or
 * *      (at your option) any later version.
 * *
 * *      This program is distributed in the hope that it will be useful,
 * *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 * *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * *      GNU General Public License for more details.
 * *      You should have received a copy of the GNU General Public License
 * *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *
 * *
 * */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>


#include "delay.h"
//#include "usart.h"
#include "sdadc.h"

#define SDADC_INIT_TIMEOUT   200 /* ~ about two SDADC clock cycles after INIT is set */
#define SDADC_CAL_TIMEOUT    (40*30720L) /*  ~5.12 ms at 6 MHz  in a single calibration sequence */


void init_all_SDADC()
{


	  SDADC_DeInit(SDADC1); //switches off vref and we could not switch it on again for some reason; even so a power cycle was needed to change vref after reflashing!!!

	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR,ENABLE);
	  PWR_SDADCAnalogCmd(PWR_SDADCAnalog_1, ENABLE);
	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
	  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SDADC1, ENABLE);
	  RCC_SDADCCLKConfig(RCC_SDADCCLK_SYSCLK_Div12); /* Set the SDADC divider: The SDADC should run max@6MHz */

	  SDADC_DeInit(SDADC2);  //dacus

	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR,ENABLE);
	  PWR_SDADCAnalogCmd(PWR_SDADCAnalog_2, ENABLE);
	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
	  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SDADC2, ENABLE);
	  RCC_SDADCCLKConfig(RCC_SDADCCLK_SYSCLK_Div12); /* Set the SDADC divider: The SDADC should run @6MHz */

	  SDADC_DeInit(SDADC3);

	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR,ENABLE);
	  PWR_SDADCAnalogCmd(PWR_SDADCAnalog_3, ENABLE);
	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);
	  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SDADC3, ENABLE);
	  RCC_SDADCCLKConfig(RCC_SDADCCLK_SYSCLK_Div12); /* Set the SDADC divider: The SDADC should run @6MHz */

	  SDADC_VREFSelect(SDADC_VREF_VREFINT2);// SDADC_VREF_Ext

	  delay_ms(5);  //5ms needed

	  //1--------------------------------------------

	  GPIO_InitTypeDef GPIO_InitStructure;

	  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
	  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AN;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_2|GPIO_Pin_14|GPIO_Pin_15;
	  GPIO_Init(GPIOB, &GPIO_InitStructure);

	  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOE, ENABLE);
	  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AN;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8|GPIO_Pin_9;
	  GPIO_Init(GPIOE, &GPIO_InitStructure);

      RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOD, ENABLE);
	  GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AN;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8;
	  GPIO_Init(GPIOD, &GPIO_InitStructure);


}





uint8_t init_SDADC1(uint32_t Channel, uint32_t Gain)
{
 uint32_t SDADCTimeout = 0;

 SDADC_Cmd(SDADC1, ENABLE);

 	   /* Enter initialization mode */
 	   SDADC_InitModeCmd(SDADC1, ENABLE);
 	   SDADCTimeout = SDADC_INIT_TIMEOUT;
 	   /* wait for INITRDY flag to be set */
 	   while((SDADC_GetFlagStatus(SDADC1, SDADC_FLAG_INITRDY) == RESET) && (--SDADCTimeout != 0));

 	   if(SDADCTimeout == 0)
 	   {
 	     /* INITRDY flag can not set */
 	     //return 1;
 	   }

SDADC_InitTypeDef SDADC_InitStruct;
SDADC_InitStruct.SDADC_Channel =Channel ;
SDADC_InitStruct.SDADC_ContinuousConvMode = DISABLE;
SDADC_InitStruct.SDADC_FastConversionMode = DISABLE;
SDADC_Init(SDADC1, &SDADC_InitStruct);

SDADC_AINStructTypeDef SDADC_AINStructure;
SDADC_AINStructInit(&SDADC_AINStructure);
SDADC_AINStructure.SDADC_InputMode = SDADC_InputMode_SEOffset;//SDADC_InputMode_SEZeroReference;//SDADC_InputMode_Diff; //SEZeroReference;
SDADC_AINStructure.SDADC_Gain = Gain;
SDADC_AINStructure.SDADC_CommonMode = SDADC_CommonMode_VSSA;
SDADC_AINStructure.SDADC_Offset = 0;
SDADC_AINInit(SDADC1, SDADC_Conf_0, &SDADC_AINStructure);


SDADC_ChannelConfig(SDADC1, Channel, SDADC_Conf_0);


SDADC_InitModeCmd(SDADC1, DISABLE);

SDADC_CalibrationSequenceConfig(SDADC1, SDADC_CalibrationSequence_1);
SDADC_StartCalibration(SDADC1);
SDADCTimeout = SDADC_CAL_TIMEOUT;
while((SDADC_GetFlagStatus(SDADC1, SDADC_FLAG_EOCAL) == RESET) && (--SDADCTimeout != 0));
if(SDADCTimeout == 0)
{
   return 2;
}

   return 0;



}


uint8_t init_SDADC2(uint32_t Channel, uint32_t Gain)
{
  uint32_t SDADCTimeout = 0;

  SDADC_Cmd(SDADC2, ENABLE);


	    /* Enter initialization mode */
	    SDADC_InitModeCmd(SDADC2, ENABLE);
	    SDADCTimeout = SDADC_INIT_TIMEOUT;
	    /* wait for INITRDY flag to be set */
	    while((SDADC_GetFlagStatus(SDADC2, SDADC_FLAG_INITRDY) == RESET) && (--SDADCTimeout != 0));

	    if(SDADCTimeout == 0)
	    {
	      /* INITRDY flag can not set */
	     // return 1;
	    }


SDADC_InitTypeDef SDADC_InitStruct;
SDADC_InitStruct.SDADC_Channel =Channel ;
SDADC_InitStruct.SDADC_ContinuousConvMode = DISABLE;
SDADC_InitStruct.SDADC_FastConversionMode = DISABLE;
SDADC_Init(SDADC2, &SDADC_InitStruct);


  SDADC_AINStructTypeDef SDADC_AINStructure;
  SDADC_AINStructInit(&SDADC_AINStructure);
  SDADC_AINStructure.SDADC_InputMode = SDADC_InputMode_Diff;//SDADC_InputMode_SEOffset;
  SDADC_AINStructure.SDADC_Gain = Gain;
  SDADC_AINStructure.SDADC_CommonMode = SDADC_CommonMode_VSSA;
  SDADC_AINStructure.SDADC_Offset = 0;
  SDADC_AINInit(SDADC2, SDADC_Conf_0, &SDADC_AINStructure);
SDADC_ChannelConfig(SDADC2, Channel, SDADC_Conf_0);

SDADC_InitModeCmd(SDADC2, DISABLE);


	     SDADC_CalibrationSequenceConfig(SDADC2, SDADC_CalibrationSequence_1);
	       SDADC_StartCalibration(SDADC2);
	       SDADCTimeout = SDADC_CAL_TIMEOUT;
	       while((SDADC_GetFlagStatus(SDADC2, SDADC_FLAG_EOCAL) == RESET) && (--SDADCTimeout != 0));
	       if(SDADCTimeout == 0)
	       {
	         return 2;
	       }



  return 0;
}




uint8_t init_SDADC3(uint32_t Channel, uint32_t Gain)
{
  uint32_t SDADCTimeout = 0;

  SDADC_Cmd(SDADC3, ENABLE);

	     /* Enter initialization mode */
	     SDADC_InitModeCmd(SDADC3, ENABLE);
	     SDADCTimeout = SDADC_INIT_TIMEOUT;
	     /* wait for INITRDY flag to be set */
	     while((SDADC_GetFlagStatus(SDADC3, SDADC_FLAG_INITRDY) == RESET) && (--SDADCTimeout != 0));

	     if(SDADCTimeout == 0)
	     {
	       /* INITRDY flag can not set */
	      // return 1;
	     }

SDADC_InitTypeDef SDADC_InitStruct;
SDADC_InitStruct.SDADC_Channel =Channel ;
SDADC_InitStruct.SDADC_ContinuousConvMode = DISABLE;
SDADC_InitStruct.SDADC_FastConversionMode = DISABLE;
SDADC_Init(SDADC3, &SDADC_InitStruct);


  SDADC_AINStructTypeDef SDADC_AINStructure;
  SDADC_AINStructInit(&SDADC_AINStructure);
  SDADC_AINStructure.SDADC_InputMode = SDADC_InputMode_SEOffset;//SDADC_InputMode_SEZeroReference;//SDADC_InputMode_SEZeroReference;
  SDADC_AINStructure.SDADC_Gain = Gain;
  SDADC_AINStructure.SDADC_CommonMode = SDADC_CommonMode_VSSA;
  SDADC_AINStructure.SDADC_Offset = 0;
  SDADC_AINInit(SDADC3, SDADC_Conf_0, &SDADC_AINStructure);
SDADC_ChannelConfig(SDADC3, Channel, SDADC_Conf_0);

SDADC_InitModeCmd(SDADC3, DISABLE);

  SDADC_CalibrationSequenceConfig(SDADC3, SDADC_CalibrationSequence_1);
  SDADC_StartCalibration(SDADC3);
  SDADCTimeout = SDADC_CAL_TIMEOUT;
  while((SDADC_GetFlagStatus(SDADC3, SDADC_FLAG_EOCAL) == RESET) && (--SDADCTimeout != 0));
  if(SDADCTimeout == 0)
  {
    return 2;
  }

  return 0;
}
